#include "polygonitem.h"

PolygonItem::PolygonItem()
{
    //
    p1=QPointF(0,0);
    p2=QPointF(20,20);
    p3=QPointF(20,180);
    p4=QPointF(0,200);

    restructPolygon(); // 

    this->setBrush(QBrush(Qt::gray));
    this->setPen(Qt::NoPen);
    this->setAcceptHoverEvents(1);

    findRotationCenter(); //  
}


void PolygonItem::findRotationCenter()
{
    QPointF leftSideCenter = p1+(p4-p1)/2; //  
    QPointF rightSideCenter = p2+(p3-p2)/2; //  
    QPointF polygonCenter = leftSideCenter + (rightSideCenter - leftSideCenter)/2;
    this->setTransformOriginPoint(polygonCenter);
    //,      ,   -  
}


void PolygonItem::restructPolygon()
{
    QPolygonF polygon;
    polygon<<p1
           <<p2
           <<p3
           <<p4;
    setPolygon(polygon);
}






void PolygonItem::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    if( this->cursor().shape() == Qt::OpenHandCursor ) {
        this->setCursor(Qt::ClosedHandCursor);
        lastMousePos=event->pos();
    }
    else lastMousePos=event->pos();
}

void PolygonItem::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
    QPointF p = event->pos()-lastMousePos; // 
    if( this->cursor().shape() == Qt::ClosedHandCursor ) { //   
       switch(pointToMove)
       {
       case 1: p1+=p; break;
       case 2: p2+=p; break;
       case 3: p3+=p; break;
       case 4: p4+=p; break;
       default: break;
       }
    }
    else if ( this->cursor().shape() == Qt::SizeAllCursor ) { //  
        p1+=p;
        p2+=p;
        p3+=p;
        p4+=p;
    }
    else if ( this->cursor().shape() == Qt::SizeBDiagCursor ) { //
        QPointF rotationP; //,   
        if ( isitTopRound ) //isitTopRound   hoverEvent
            rotationP = p1+(p2-p1)/2; // 
        else
            rotationP = p4+(p3-p4)/2; // 
        rotateItem( rotationP, p);
        acceptRotation();
    }
    restructPolygon();
    lastMousePos=event->pos();
}


void PolygonItem::rotateItem(const QPointF& rotationP, const QPointF& mouseP)
{
    QPointF center = this->transformOriginPoint(); //   
    qreal cx = center.x();
    qreal cy = center.y();
    qreal rx = rotationP.x();
    qreal ry = rotationP.y();
    qreal x = mouseP.x();
    qreal y = mouseP.y();

    //,    rotationP  center,   
    //          -
    //    6  :)

    if ( rx >= cx && ry <= cy ) {       //  0-3 (  )
        if (x > 0 && y < 0) return;     //   - 1:30, 
        if ( abs(x) > abs(y) )          // 
             this->setRotation(this->rotation()+x);
        else
             this->setRotation(this->rotation()+y);
    }
    else if ( rx >= cx && ry >= cy ) {      //  3-6
         if ( x > 0 && y > 0 ) return;      //   - 4:30, 
         if ( abs(x) > abs(y) )
              this->setRotation(this->rotation()-x);
         else
              this->setRotation(this->rotation()+y);
    }
    else if ( rx <= cx && ry >= cy ) {      //  6-9
         if ( x < 0 && y > 0 ) return;      //   - 7:30, 
         if ( abs(x) > abs(y) )
              this->setRotation(this->rotation()-x);
         else
              this->setRotation(this->rotation()-y);
    }
    else if ( rx <= cx && ry <= cy ) {      //  9-0
         if ( x < 0 && y < 0 ) return;      //   - 10:30, 
         if ( abs(x) > abs(y) )
              this->setRotation(this->rotation()+x);
         else
              this->setRotation(this->rotation()-y);
    }
}


void PolygonItem::acceptRotation()
{
    //   ,       ,    
    //   ,    ,     
    //            

    p1=this->mapToScene(this->polygon().at(0));
    p2=this->mapToScene(this->polygon().at(1));
    p3=this->mapToScene(this->polygon().at(2));
    p4=this->mapToScene(this->polygon().at(3));
    this->setRotation(0);
}


void PolygonItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
    findRotationCenter(); //    ,    
    this->setCursor(Qt::OpenHandCursor);
}







void PolygonItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
    QPointF pos = event->pos();
    //  -     
    if ( isMouseAroundPoint(pos,p1) ) {
        pointToMove=1;
        this->setCursor(Qt::OpenHandCursor);
    }
    else if ( isMouseAroundPoint(pos,p2) ) {
        pointToMove=2;
        this->setCursor(Qt::OpenHandCursor);
    }
    else if ( isMouseAroundPoint(pos,p3) ) {
        pointToMove=3;
        this->setCursor(Qt::OpenHandCursor);
    }
    else if ( isMouseAroundPoint(pos,p4) ) {
        pointToMove=4;
        this->setCursor(Qt::OpenHandCursor);
    }

    //  -    " " 
    else if ( isMouseAroundPoint(pos,p1+(p2-p1)/2) ) {
        isitTopRound=1; // 
        this->setCursor(Qt::SizeBDiagCursor);
    }
    else if ( isMouseAroundPoint(pos,p4+(p3-p4)/2) ) {
        isitTopRound=0; // 
        this->setCursor(Qt::SizeBDiagCursor);
    }

    //    
    else
        this->setCursor(Qt::SizeAllCursor);
}


bool PolygonItem::isMouseAroundPoint(const QPointF& hoverEnterPoint, const QPointF& center)
{
    qreal x = hoverEnterPoint.x();
    qreal y = hoverEnterPoint.y();
    qreal px = center.x();
    qreal py = center.y();

    //+5  -5    
    if ( x < px+5
      && x >= px-5
      && y < py+5
      && y >= py-5
      )
       return true;
    else return false;
}


void PolygonItem::hoverMoveEvent(QGraphicsSceneHoverEvent *event)
{
    hoverEnterEvent(event);
    //         
    // "" ,     
    //  ,   hoverEnterEvent,     
}


void PolygonItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
    this->setCursor(Qt::ArrowCursor);
}
